from docutils import nodes
from sphinx import addnodes
from sphinx.directives import ObjectDescription
from sphinx.domains import Domain, ObjType
from sphinx.roles import XRefRole
from sphinx.util import ws_re
from sphinx.util.nodes import make_refnode


class ConfigOption(ObjectDescription):
    indextemplate = "configuration option; %s"
    parse_node = None

    def handle_signature(self, sig, signode):
        if self.parse_node:
            name = self.parse_node(self.env, sig, signode)
        else:
            signode.clear()
            signode += addnodes.desc_name(sig, sig)
            # normalize whitespace like XRefRole does
            name = ws_re.sub("", sig)
        return name

    def add_target_and_index(self, name, sig, signode):
        targetname = f"{self.objtype}-{name}"
        signode["ids"].append(targetname)
        self.state.document.note_explicit_target(signode)
        if self.indextemplate:
            colon = self.indextemplate.find(":")
            if colon != -1:
                indextype = self.indextemplate[:colon].strip()
                indexentry = self.indextemplate[colon + 1 :].strip() % (name,)
            else:
                indextype = "single"
                indexentry = self.indextemplate % (name,)
            self.indexnode["entries"].append(
                (indextype, indexentry, targetname, targetname, None),
            )
        self.env.domaindata["config"]["objects"][self.objtype, name] = (
            self.env.docname,
            targetname,
        )


class ConfigSectionXRefRole(XRefRole):
    """Cross-referencing role for configuration sections (adds an index entry)."""

    def result_nodes(self, document, env, node, is_ref):
        if not is_ref:
            return [node], []
        varname = node["reftarget"]
        tgtid = "index-{}".format(env.new_serialno("index"))
        indexnode = addnodes.index()
        indexnode["entries"] = [
            ("single", varname, tgtid, varname, None),
            ("single", f"configuration section; {varname}", tgtid, varname, None),
        ]
        targetnode = nodes.target("", "", ids=[tgtid])
        document.note_explicit_target(targetnode)
        return [indexnode, targetnode, node], []


class ConfigSection(ObjectDescription):
    indextemplate = "configuration section; %s"
    parse_node = None

    def handle_signature(self, sig, signode):
        if self.parse_node:
            name = self.parse_node(self.env, sig, signode)
        else:
            signode.clear()
            signode += addnodes.desc_name(sig, sig)
            # normalize whitespace like XRefRole does
            name = ws_re.sub("", sig)
        return name

    def add_target_and_index(self, name, sig, signode):
        targetname = f"{self.objtype}-{name}"
        signode["ids"].append(targetname)
        self.state.document.note_explicit_target(signode)
        if self.indextemplate:
            colon = self.indextemplate.find(":")
            if colon != -1:
                indextype = self.indextemplate[:colon].strip()
                indexentry = self.indextemplate[colon + 1 :].strip() % (name,)
            else:
                indextype = "single"
                indexentry = self.indextemplate % (name,)
            self.indexnode["entries"].append(
                (indextype, indexentry, targetname, targetname, None),
            )
        self.env.domaindata["config"]["objects"][self.objtype, name] = (
            self.env.docname,
            targetname,
        )


class ConfigOptionXRefRole(XRefRole):
    """Cross-referencing role for configuration options (adds an index entry)."""

    def result_nodes(self, document, env, node, is_ref):
        if not is_ref:
            return [node], []
        varname = node["reftarget"]
        tgtid = "index-{}".format(env.new_serialno("index"))
        indexnode = addnodes.index()
        indexnode["entries"] = [
            ("single", varname, tgtid, varname, None),
            ("single", f"configuration option; {varname}", tgtid, varname, None),
        ]
        targetnode = nodes.target("", "", ids=[tgtid])
        document.note_explicit_target(targetnode)
        return [indexnode, targetnode, node], []


class ConfigFileDomain(Domain):
    name = "config"
    label = "Config"

    object_types = {
        "option": ObjType("config option", "option"),
        "section": ObjType("config section", "section"),
    }
    directives = {
        "option": ConfigOption,
        "section": ConfigSection,
    }
    roles = {
        "option": ConfigOptionXRefRole(),
        "section": ConfigSectionXRefRole(),
    }

    initial_data = {
        "objects": {},  # (type, name) -> docname, labelid
    }

    def clear_doc(self, docname):
        remove = []
        for key, (fn, _) in self.data["objects"].items():
            if fn == docname:
                remove.append(key)
        for key in remove:
            del self.data["objects"][key]

    def resolve_xref(self, env, fromdocname, builder, typ, target, node, contnode):
        docname, labelid = self.data["objects"].get((typ, target), ("", ""))
        if not docname:
            return None
        return make_refnode(builder, fromdocname, docname, labelid, contnode)

    def get_objects(self):
        for (type, name), info in self.data["objects"].items():
            yield (
                name,
                name,
                type,
                info[0],
                info[1],
                self.object_types[type].attrs["searchprio"],
            )


def setup(app):
    app.add_domain(ConfigFileDomain)



#!/usr/bin/python
# vim: expandtab sw=4 ts=4 sts=4:
"""
Script to decode USB Snoop Pro log to human readable form.

This is probably only useful for AT commands.
"""

import sys

if len(sys.argv) < 2:
    print("Usage: decode.py LOGFILE")
    sys.exit(1)

with open(sys.argv[1]) as f:
    output = ""
    for line in f:
        if line[:14] == "<payloadbytes>":
            line = line[14:-17]
            print(line)
            print(line.decode("hex"))



#!/usr/bin/env python
# vim: expandtab sw=4 ts=4 sts=4:
"""Gammu SMS backup generator."""

__author__ = "Michal Čihař"
__email__ = "michal@cihar.com"
__license__ = """
Copyright © 2003 - 2018 Michal Čihař

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""

import os

# Work in both common location when this can be executed:
try:
    os.chdir("tests/at-sms-encode/")
except OSError:
    os.chdir("at-sms-encode/")

# Numbers we're going to test
NUMBERS = [
    "1234",
    "800123456",
    "+420800123456",
    "+41761234567",
]

# Text parts we're going to test
TEXTS = [
    "123456",
    "Zkouška sirén",
    "This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License version 2 as published by the Free Software Foundation.",
]

TEMPLATE = """
[SMSBackup000]
SMSC = "%s"
State = %s
Number = "%s"
Coding = %s
Folder = %d
"""

STATES = [
    "Read",
    "Read",
    "Sent",
]
CODINGS = [
    "Default",
    "Unicode",
]


def write_text(f, text):
    """Writes text splitted and encoded in same way as Gammu does it for SMS backups."""
    encoded = text.encode("UTF-16-BE").encode("HEX")
    line = 0
    while len(encoded) > 0:
        f.write("Text%02d = %s\n" % (line, encoded[:200]))
        encoded = encoded[200:]
        line = line + 1


def generate_message(index, folder, coding, smscnum, num, text):
    """Generates single message file."""
    f = file("%02d.backup" % index, "w")
    f.write(
        TEMPLATE
        % (NUMBERS[smscnum], STATES[folder], NUMBERS[num], CODINGS[coding], folder),
    )
    if folder > 1:
        f.write("Sent = 20070605T135630\n")
    write_text(f, TEXTS[text])
    f.close()


def generate():
    """Generates test data based on NUMBERS and TEXTS variables."""
    index = 1

    for smscnum in range(len(NUMBERS)):
        for num in range(len(NUMBERS)):
            for text in range(len(TEXTS)):
                for coding in range(len(CODINGS)):
                    for folder in [1, 2]:
                        generate_message(index, folder, coding, smscnum, num, text)
                        index = index + 1


if __name__ == "__main__":
    generate()
